#!/usr/bin/perl

use warnings;
use strict;

sub BEGIN
{
    push (@INC,$ENV{HOME}.'/download/leverloda/goanseech')
        if(!( (join(' ',@INC)) =~ /$ENV{HOME}\/\.leve\/lib/));
}

# Gtk support is enabled if the Gtk module is used before POE itself.

#use Gtk;
use Gnome;

use POE;

use Data::Dumper;
use POE::Filter::Reference;
use POE::Component::Client::TCP;

##############################################################################
=pod

=head1 NAME

gtkGUI connects to a patternServer and allows to manipulate the data of
that server

=head1 AUTHOR

  Bruno Bttcher <bboett at adlp dot org>

=head1 COPYRIGHT and LICENCE

 Copyright (c) 2002 Bruno Boettcher

 gtkGUI.pl is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


=head3 instantiating a TCP client

Create a client that sends and receives Perl data structures.
Upon connecting with the server, it sends a list of things to
process.  The server responds with a hash of a few things.  Upon
receipt of that hash, the client displays its contents and exits.

=cut

##############################################################################
init Gnome "Goanseech";

my $app = new Gnome::App "Goanseech", "Leech system! Version";

my $rootpath = $ENV{HOME}."/.goanseech/";


while(scalar(@ARGV) >0)
{
  my $key = shift(@ARGV);
  if($key =~ /--config/i)
  {
    $rootpath = shift(@ARGV);
  }# if($key =~ /--config/i)
  else
  {
    die("Invalid switch: valid is --config\n");
  }# else
}# if(scalar(@ARGV) >0)
$rootpath .= "/" if(!($rootpath =~ /\/$/));

my $settings = do($rootpath."settings.pm");

my %series = ();

my $server;

my $table = [];
my $buildup = 1;
my $NAME = "Goanseech";

#make the connection to the patternServer and retrieve the series list
use ClientTcp;
my $clientSocket = 
ClientTcp->new
( 
 Alias => "gtkGUI",
 RemoteAddress => "localhost",
 RemotePort    => 22345,
 Filter        => "POE::Filter::Reference",
 SSL => 1,

 # Build a request and send it.

 Connected => sub {
   my $heap = $_[HEAP];

   my @request = ( 
       "auth ".$settings->{"user"}." ".$settings->{"passwd"},
       "series" 
       );

   $heap->{server}->put( \@request );
   $server = $heap->{server};
   #print("shell initialized send out: ".Dumper(\@request)."\n");
   #prompt();
 },

# Receive a response, display it, and shut down the client.

ServerInput => sub {
  my ( $kernel, $hash, $heap ) = @_[ KERNEL, ARG0, HEAP ];

  #$kernel->post('delegate',"hello");
  #print("got input....!\n");

  if ( !$hash) 
  {
    #ignore empty lines...
  }
  elsif ( ref($hash) =~ /nameContainer/ ) 
  {
    #print ("Client received a nameContainer type: ". ref($hash). " with $hash\n");
    #print ("dumping: ". Dumper($hash). "\n");
    #print "Client received:".Dumper($hash)."\n";
    #print ("Client received:".Dumper($hash)."\n");
    my $list = $hash->{"listing"};
    print ("listing:".Dumper($list)."\n");
  }
  elsif ( ref($hash) eq 'HASH' ) 
  {
    #print "Client received:".Dumper($hash)."\n";
    #print ("Client received:".Dumper($hash)."\n");
    print (Dumper($hash)."\n");
  }
  elsif ( ref($hash) eq 'SCALAR' ) 
  {
    #print "Client received:".$$hash."\n";
    if($$hash =~ /^\s*---- Series/i)
    {
      #got the series listing!
      my @rawseries = split(/\n/,$$hash);
      #remove enclosing rubbish
      shift(@rawseries);
      pop(@rawseries);
      foreach my $line (@rawseries)
      {
	if( $line =~ /.*?'(.*?)'(\d+)-(\d+)\s+other:\s+(.*)/)
	{
	  my $entry = {
	    "name" => $1, "min" => $2, "max" => $3,
	    "other" => $4
	  };
	  $series{$1} = $entry;
	}# if( $line =~ /.*?'(.*?)'(\d+)-(\d+)\s+other:\s+(.*)/)
	elsif( $line =~ /.*?'(.*?)'(\d+)-(\d+)/)
	{
	  my $entry = {
	    "name" => $1, "min" => $2, "max" => $3
	  };
	  #push(@series,$entry);
	  $series{$1} = $entry;
	}#else
	else
	{
	  print("unparsable: $line\n");
	}# else

      }# foreach my $line (@rawseries)
      #print("series: ".Dumper(\@series)."\n");
      $_[KERNEL]->post("gtkSeries", "fillup"); 
    }# if($$hash =~ /^\s*---- Series/i)
    elsif($$hash =~ /jobDone/i){}
    else
    {
      print ("sc: ".$$hash."\n");
      if($heap->{main_window})
      {
	$heap->{main_window}->message_dialog($$hash);
      }# if(heap->{main_window})
    }
  }
  else 
  {
    #print "Client received an unknown response type: ", ref($hash), " with $hash\n";
    print ("Client received an unknown response type: ". ref($hash). " with $hash\n");
  }

  #$kernel->yield("shutdown");
},
#  #could reach the server....
#  ConnectError => sub {
  #    my ($heap,$kernel) = @_[HEAP,KERNEL];

  #print("no socket generated!!\n");
  #print("trying ot instantiate a server\n");
  #system("nohup ./patternServer.pl &\n");
  #print("patternserver instantiated and running\n");
  #$_[KERNEL]->delay( reconnect => 5 );
  #  },
#   Disconnected => sub {
  #     my ($heap,$kernel, $this) = @_[HEAP,KERNEL,OBJECT];
  #     print ("The server disconnected... from this ".ref($this)."\n");
  #     my $msg = "reconnecting in 5 secs if no instance is found starting a new one";
  #     $msg .= "\n";
  #     print ($msg);
  # #system("nohup ./patternServer.pl &\n");
  # $_[KERNEL]->delay( reconnect => 5 );
  #   },
);

# Create the session that will drive the user interface.

POE::Session->create
( inline_states =>
  { _start => \&ui_start,
  ev_count => \&ui_count,
  ev_clear => \&ui_clear,
  ev_change => \&ui_change,
  ev_delete => \&ui_delete,
  fillup => \&create_window,
  }
);

# Run the program until it is exited.

$poe_kernel->run();
exit 0;

##############################################################################
=pod

=item _start

to keep this session alive set an alias

=cut
######################################################################
sub ui_start
{
  my ( $kernel, $arg ) = @_[ KERNEL, ARG0 ];
  $_[KERNEL]->alias_set("gtkSeries");
  #$kernel->yield("fetchURLs");  
}# sub ui_start
##############################################################################
=pod

=item create_window 

create the user interface when the session starts.  This assumes
some familiarity with Gtk.  ui_start() illustrates four important
points.

- Gtk events do not require require a main window.  It is therefore
the responsibility of the programmer to create a main window (or
not) herself.  POE::Kernel's signal_ui_destroy() method attaches a
IDESTROY signal to the destruction of a window.  In this case,
closing the main window signals the program to shut down.

- Widgets we need to work with later, such as the counter display,
must be stored somewhere.  The heap is a convenient place for them.

- Gtk widgets expect callbacks in the form of coderefs.  The
session's postback() method provides coderefs that post events when
called.  The Button created in ui_start() fires an "ev_clear" event
when it is pressed.

- POE::Kernel methods such as yield(), post(), delay(), signal(),
and select() (among others) work the same as they would without Gtk.
this feature makes it possible to write back end sessions that
support multiple GUIs with a single code base.

=cut

##############################################################################
sub create_window 
{
  my ( $kernel, $session, $heap ) = @_[ KERNEL, SESSION, HEAP ];

  #init Gnome "Goanseech";

  #my $app = new Gnome::App "Goanseech", "Leech system! Version";

  $heap->{main_window} = $app;

  #show_all($app);
  $app->show();
  signal_connect $app 'delete_event',
                           sub { Gtk->main_quit; return 0 };

  $kernel->signal_ui_destroy( $app );
  #$heap->{main_window}->set_usize(450, 500 );

  #$heap->{scroll_window} = Gtk::ScrolledWindow->new();
  #$heap->{main_window}->add($heap->{scroll_window});

  #my $viewport = new Gtk::Viewport(new Gtk::Adjustment( 0.0, 0.0, 101.0, 0.1, 1.0, 1.0), new Gtk::Adjustment( 0.0, 0.0, 101.0, 0.1, 1.0, 1.0) );;
  #$heap->{scroll_window}->add_with_viewport($viewport);
  #$viewport->show();

  if($buildup)
  {
    $buildup = 0;
    #my $fixed = new Gtk::Fixed;
    my $fixed = new Gtk::VBox(0,0);
    #$viewport->add( $fixed);
    $app->set_contents($fixed);
    #$heap->{main_window}->add( $fixed);
    #$heap->{scroll_window}->add_with_viewport( $fixed);
    $fixed->show;


$app->create_menus(
        {type => 'subtree',
         label => '_File',
         subtree => [
                {type => 'item',
                 label => '_Load',
                 callback => sub {return ui_load($kernel);}
                },
                {type => 'item',
                 label => '_Save',
                 callback => sub {return ui_save($kernel);}
                },
                {type => 'item',
                 label => '_Reload',
                 callback => sub {return ui_reload($kernel);}
                },
                {type => 'item',
                 label => 'E_xit',
                 callback => sub {ui_quit($kernel); Gtk->main_quit; return 0 }
                },
                    ]
        },
        {type => 'subtree',
         label => '_Help',

         subtree => [
                {type => 'item', 
                 label => '_About...',
                 callback => sub {return ui_about($kernel,$heap);}
                }
                    ]
        }
      );

    $heap->{scroll_window} = Gtk::ScrolledWindow->new();
    #$heap->{scroll_window}->set_usize(430, 500 );
    $fixed->add($heap->{scroll_window});
    #$fixed->pack_start($heap->{scroll_window},1,1,0);
    #$fixed->add($heap->{scroll_window});
    #$heap->{scroll_window}->set_uposition(0, 24 );

    my $box = Gtk::Table->new( 4, scalar(keys(%series)) +1 );
    #$box->set_usize(486, 100 );
    #$viewport->add($box);
    #$heap->{scroll_window}->add($viewport);
    #$heap->{scroll_window}->add_with_viewport($box);
    #$fixed->add($box);
    #$viewport->add($box);
    $heap->{scroll_window}->add_with_viewport($box);
    #$box->set_uposition(0, 24 );
    #$box->set_usize(80, 96 );
    #$heap->{scroll_window}->add(Gtk::Viewport::new($box,0,0));
    #$box->set_uposition(0, 24 );
    $box->show();

    #print("ui_start! series = ".Dumper(\@series)."\n");

    my $index = 0;
    my $next = $index+1;
    my @titles = qw(Pattern delete min max other-episodes);
    my $x = 0;
    foreach my $title (@titles)
    {
      my $label = Gtk::Label->new($title);
      Gtk::Table::attach( $box, $label, $x, ($x + 1), $index, $next, 0, 0, 0, 0);
      $label->show();
      $x++;
    }# foreach my $title (@titles)

    $index++;

    foreach my $pattern (sort(keys(%series)))
    {
      my $line = $series{$pattern};
      push(@$table, add_table_line($box,$line,$index,$kernel));
    $index++;

    }# foreach my $line (@series)

    #$heap->{counter}       = 0;
    #$heap->{counter_label} = Gtk::Label->new( $heap->{counter} );
    ##$box->pack_start( $heap->{counter_label}, 1, 1, 0 );
    #Gtk::Table::attach( $box,$heap->{counter_label}, 1, 2, 0, 1, 0, 0, 0, 0);
    #$heap->{counter_label}->show();

    #my $button = Gtk::Button->new("Clear");
    #$button->signal_connect( "clicked", $session->postback("ev_clear") );
    ##$box->pack_start( $button, 1, 1, 0 );
    #Gtk::Table::attach( $box,$button, 0, 2, 1, 2, 0, 0, 0, 0);
    #$button->show();

    $heap->{scroll_window}->show();
    $heap->{main_window}->show();

    #$kernel->yield("ev_count");
  }# if(scalar($table) <= 0)
  else
  {
    print ("reloading...\n");

    my @patterns = sort(keys(%series));
  #print("dataset: ".Dumper(\@patterns)."\n");
  print("tables: ".scalar(@patterns)." vs ".scalar(@$table)."\n");
    foreach my $table_line (@$table)
    {
      my $pattern = shift(@patterns);
      my $line = $series{$pattern};
      print("refreshing line $pattern with ".Dumper($line)."\n");
      for my $td (qw(name min max other))
      {
	if($line->{$td})
	{
	$table_line->{$td}->set_text($line->{$td});
	}# if(!$line)
	#TODO disconnect signal handler and reconnect it....
     #$label->signal_connect( 'clicked', \&delseries, $line->{"name"},$kernel);
      #$label->signal_connect( 'value_changed', \&min_change, $line->{"name"}, "max",$kernel);
      #$label->signal_connect( 'value_changed', \&min_change, $line->{"name"}, "min",$kernel);
      }# for my $td (qw(name min max other))

    }# foreach my $line (@series)
    print("finished replenishing: ".scalar(@$table)." vs ".scalar(@patterns)."\n");
     #foreach my $pattern (@patterns)
     #{
     #  push(@$table, add_table_line($box,$line,$index,$kernel));
     #$index++;
     #}# foreach my $line (@series)
  }# else
}

##############################################################################
=pod

=item ui_count 

Handle the "ev_count" event by increasing a counter and displaying
its new value.

=cut

##############################################################################
sub ui_count 
{
  my ( $kernel, $heap ) = @_[ KERNEL, HEAP ];
  $heap->{counter_label}->set_text( ++$_[HEAP]->{counter} );
  $kernel->yield("ev_count");
}
##############################################################################
=pod

=item ui_clear 

Handle the "ev_clear" event by clearing and redisplaying the
counter.

=cut

##############################################################################
sub ui_clear 
{
  $_[HEAP]->{counter} = 0;
}
##############################################################################
=pod

=item ui_change 

some changes were made to the boundaries of some series reflect those changes to the server

=cut

##############################################################################
sub ui_change 
{
  my($type,$pattern,$value) = @_[ARG0,ARG1,ARG2];
  #clear$_[HEAP]->{counter} = 0;
  print("ui_change: 	pat:".$_[ARG1]." field:".$_[ARG0]." value:".$_[ARG2]."\n");
    my @request;
  if($type eq "min")
  {
    if($series{$pattern}->{$type} > $value)
    {
      print("delepisode of $pattern ...\n");
      $series{$pattern}->{$type} -- ;
      @request = ( 
	  "delepisode $pattern ".$series{$pattern}->{$type},
	  );
    }# if($series{$pattern}->{$type} > $value)
    else
    {
      print("addepisode for $pattern ... \n");
      @request = ( 
	  "addepisode $pattern ".$series{$pattern}->{$type},
	  );
      $series{$pattern}->{$type} ++ ;
    }# else
  }# if($type eq "min")
  elsif($type eq "max" || $type eq "prio" || $type eq "tag")
  {
    print("modseries for $pattern ... \n");
    @request = ( 
	"modseries $pattern $type $value",
	);
    $series{$pattern}->{$type} = $value ;
  }# else
    $server->put( \@request );
}# sub ui_change 
##############################################################################
=pod

=item min_change 

callback for the GTK stuff, this method is called when an event occurs on one of the spin buttons

=cut

##############################################################################
sub min_change 
{
  my($adj,$pattern,$type,$kernel) = @_;
  #print("min_change ".Dumper(\@_)."\n");
  my $newval = $adj->get_value();
  $kernel->post("gtkSeries", "ev_change", $type, $pattern, $newval) ;
}
##############################################################################
=pod

=item delseries 

callback for the GTK stuff, this method is called when an event occurs on one of the spin buttons

=cut

##############################################################################
sub delseries 
{
  my($adj,$pattern,$kernel) = @_;
  #print("min_change ".Dumper(\@_)."\n");
  $kernel->post("gtkSeries", "ev_delete", $pattern) ;
}
##############################################################################
=pod

=item ui_quit 

close down the GUI

=cut

##############################################################################
sub ui_quit 
{
  my($obj, $kernel) = @_;
  undef $server;
  print("normal shutdown\n");

  $kernel->post("gtkGUI", "shutdown" ) ;
}
##############################################################################
=pod

=item ui_load 

tell the server to reload its files

=cut

##############################################################################
sub ui_load 
{
  my($obj, $kernel) = @_;
  print("reloading\n");

  my @request = ( 
      "load",
      );
  $server->put( \@request );
}
##############################################################################
=pod

=item ui_save 

tell the server to save its files

=cut

##############################################################################
sub ui_save 
{
  my($obj, $kernel) = @_;
  print("saving\n");

  my @request = ( 
      "save",
      );
  $server->put( \@request );
}
##############################################################################
=pod

=item ui_reload 

tell the server to resend the listing

=cut

##############################################################################
sub ui_reload 
{
  my($obj, $kernel) = @_;
  print("reloading\n");

  my @request = ( 
      "series",
      );
  $server->put( \@request );
}
##############################################################################
=pod

=item ui_delete 

tell the server to delete a series

=cut

##############################################################################
sub ui_delete 
{
  my($obj, $kernel,$pattern) = @_[OBJECT,KERNEL, ARG0];
  #my($adj,$pattern,$type,$kernel) = @_;
  #print("min_change ".Dumper(\@_)."\n");

  my @request = ( 
      "delseries $pattern",
      );
  $server->put( \@request );
}
##############################################################################
=pod

=item ui_about 

generate an about message

=cut

##############################################################################
sub ui_about 
{
  my($obj, $kernel,$heap) = @_;
  print("about\n");


  my $about = new Gnome::About $NAME, "v1.0",
  "(C) Bruno Bttcher, 2003", ["Bruno Bttcher"],
  "This program is released under the GPL";
  show $about;

}
##############################################################################
=pod

=item add_table_line 

add a line to the actual table

=cut

##############################################################################
sub add_table_line
{
  my ($box, $line, $index,$kernel) = @_;
      my $table_line = {};
      my $next = $index + 1;

      my $x = 0;
      my $label = Gtk::Label->new($line->{"name"});
      $label->set_justify('left');
      $label->set_line_wrap(0 );
      #$box->table_attach( $label, 0, 1, 0, 1,0,0,0,0);
      $next = $index+1;
      Gtk::Table::attach( $box, $label, $x, ($x + 1), $index, $next, 0, 0, 0, 0);
      $label->show();
      $table_line->{"name"} = $label;
      $x++;

      $label = Gtk::Button->new('D');
      $next = $index+1;
      Gtk::Table::attach( $box, $label, $x, ($x + 1), $index, $next, 0, 0, 0, 0);
      $label->signal_connect( 'clicked', \&delseries, $line->{"name"},$kernel);
      $table_line->{"delete"} = $label;
      $label->show();
      $x++;

      #$label = Gtk::Entry->new();
      #$label->set_text($line->{"min"});
      my $adj = new Gtk::Adjustment($line->{"min"}, 0, 1025, 1, 10, 10 );
      $label = new Gtk::SpinButton($adj,1,0);
      $adj->signal_connect( 'value_changed', \&min_change, $line->{"name"}, "min",$kernel);
      #$adj->signal_connect( 'value_changed', sub { $kernel->post("gtkSeries", "ev_change") }, $line->{"name"}, "min", $label->get_value_as_int());
      Gtk::Table::attach( $box, $label, $x, ($x + 1), $index, $next, 0, 0, 0, 0);
      $label->show();
      $table_line->{"min"} = $label;
      $x++;

      $adj = new Gtk::Adjustment($line->{"max"}, 0, 1025, 1, 10, 10 );
      $label = new Gtk::SpinButton($adj,1,0);
      $adj->signal_connect( 'value_changed', \&min_change, $line->{"name"}, "max",$kernel);
      Gtk::Table::attach( $box, $label, $x, ($x + 1), $index, $next, 0, 0, 0, 0);
      $label->show();
      $table_line->{"max"} = $label;
      $x++;

      my $text = " ";
      $text = $line->{"other"} if($line->{"other"});
      #$label = new Gtk::Text( undef, undef );
      #$label->set_editable(1 );
      #$label->insert(undef, undef, undef, $text);
      $label = new Gtk::Label($text);
      $label->set_line_wrap(1 );
      #$label->set_editable(1 );
      Gtk::Table::attach( $box, $label, $x, ($x + 1), $index, $next, 0, 0, 0, 0);
      $label->show();
      $table_line->{"other"} = $label;
      $x++;

      #print("added ".$line->{"name"}." at ".$index."x$next\n");
      return($table_line);
}#sub add_table_line

__END__


=back

=head1 THANKS

I. Wronsky came up with the concept for leverloda
This gave B. Boettcher the incentive to begin this work, the
famous itching paradigm!

=head1 SEE ALSO

http://www.geocities.com/iwronsky/
http://poe.perl.org/

=cut

